package com.boman.system.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import com.boman.common.core.constant.Constants;
import com.boman.common.core.utils.DateUtils;
import com.boman.common.core.web.domain.AjaxResult;
import com.boman.gen.domain.GenTable;
import com.boman.gen.domain.GenTableColumn;
import com.boman.gen.mapper.GenTableColumnMapper;
import com.boman.gen.mapper.GenTableMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.boman.common.core.constant.UserConstants;
import com.boman.common.core.utils.SecurityUtils;
import com.boman.common.core.utils.StringUtils;
import com.boman.system.api.domain.SysRole;
import com.boman.system.api.domain.SysUser;
import com.boman.system.domain.SysMenu;
import com.boman.system.domain.vo.MetaVo;
import com.boman.system.domain.vo.RouterVo;
import com.boman.system.domain.vo.TreeSelect;
import com.boman.system.mapper.SysMenuMapper;
import com.boman.system.mapper.SysRoleMapper;
import com.boman.system.mapper.SysRoleMenuMapper;
import com.boman.system.service.ISysMenuService;

/**
 * 菜单 业务层处理
 * 
 * @author ruoyi
 */
@Service
public class SysMenuServiceImpl implements ISysMenuService
{
    public static final String PREMISSION_STRING = "perms[\"{0}\"]";

    @Autowired
    private SysMenuMapper menuMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysRoleMenuMapper roleMenuMapper;

    @Autowired
    private GenTableColumnMapper genTableColumnMapper;

    @Autowired
    private GenTableMapper genTableMapper;

    /**
     * 根据用户查询系统菜单列表
     * 
     * @param userId 用户ID
     * @return 菜单列表
     */
    @Override
    public List<SysMenu> selectMenuList(Long userId)
    {
        return selectMenuList(new SysMenu(), userId);
    }

    /**
     * 查询系统菜单列表
     * 
     * @param menu 菜单信息
     * @return 菜单列表
     */
    @Override
    public List<SysMenu> selectMenuList(SysMenu menu, Long userId)
    {
        List<SysMenu> menuList = null;
        // 管理员显示所有菜单信息
        if (SysUser.isAdmin(userId))
        {
            menuList = menuMapper.selectMenuList(menu);
        }
        else
        {
            menu.getParams().put("userId", userId);
            menuList = menuMapper.selectMenuListByUserId(menu);
        }
        return menuList;
    }

    /**
     * 根据用户ID查询权限
     * 
     * @param id 用户ID
     * @return 权限列表
     */
    @Override
    public Set<String> selectMenuPermsById(Long id)
    {
        List<String> perms = menuMapper.selectMenuPermsById(id);
        Set<String> permsSet = new HashSet<>();
        for (String perm : perms)
        {
            if (StringUtils.isNotEmpty(perm))
            {
                permsSet.addAll(Arrays.asList(perm.trim().split(",")));
            }
        }
        return permsSet;
    }

    /**
     * 根据用户ID查询菜单
     * 
     * @param id 用户名称
     * @return 菜单列表
     */
    @Override
    public List<SysMenu> selectMenuTreeById(Long id)
    {
        List<SysMenu> menus = null;
        if (SecurityUtils.isAdmin(id))
        {
            menus = menuMapper.selectMenuTreeAll();
        }
        else
        {
            menus = menuMapper.selectMenuTreeById(id);
        }
        return getChildPerms(menus, 0);
    }

    /**
     * 根据角色ID查询菜单树信息
     * 
     * @param roleId 角色ID
     * @return 选中菜单列表
     */
    @Override
    public List<Integer> selectMenuListByRoleId(Long roleId)
    {
        SysRole role = roleMapper.selectRoleById(roleId);
        return menuMapper.selectMenuListByRoleId(roleId, role.isMenuCheckStrictly());
    }

    /**
     * 构建前端路由所需要的菜单
     * 
     * @param menus 菜单列表
     * @return 路由列表
     */
    @Override
    public List<RouterVo> buildMenus(List<SysMenu> menus)
    {
        List<RouterVo> routers = new LinkedList<RouterVo>();
        for (SysMenu menu : menus)
        {
            RouterVo router = new RouterVo();
            router.setHidden("1".equals(menu.getVisible()));
            router.setName(getRouteName(menu));
            router.setPath(getRouterPath(menu));
            router.setComponent(getComponent(menu));
            router.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon(), StringUtils.equals("1", menu.getIsCache())));
            List<SysMenu> cMenus = menu.getChildren();
            if (!cMenus.isEmpty() && cMenus.size() > 0 && UserConstants.TYPE_DIR.equals(menu.getMenuType()))
            {
                router.setAlwaysShow(true);
                router.setRedirect("noRedirect");
                router.setChildren(buildMenus(cMenus));
            }
            else if (isMeunFrame(menu))
            {
                List<RouterVo> childrenList = new ArrayList<RouterVo>();
                RouterVo children = new RouterVo();
                children.setPath(menu.getPath());
                children.setComponent(menu.getComponent());
                children.setName(StringUtils.capitalize(menu.getPath()));
                children.setMeta(new MetaVo(menu.getMenuName(), menu.getIcon(), StringUtils.equals("1", menu.getIsCache())));
                childrenList.add(children);
                router.setChildren(childrenList);
            }
            routers.add(router);
        }
        return routers;
    }

    /**
     * 构建前端所需要树结构
     * 
     * @param menus 菜单列表
     * @return 树结构列表
     */
    @Override
    public List<SysMenu> buildMenuTree(List<SysMenu> menus)
    {
        List<SysMenu> returnList = new ArrayList<SysMenu>();
        List<Long> tempList = new ArrayList<Long>();
        for (SysMenu dept : menus)
        {
            tempList.add(dept.getId());
        }
        for (Iterator<SysMenu> iterator = menus.iterator(); iterator.hasNext();)
        {
            SysMenu menu = (SysMenu) iterator.next();
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(menu.getParentId()))
            {
                recursionFn(menus, menu);
                returnList.add(menu);
            }
        }
        if (returnList.isEmpty())
        {
            returnList = menus;
        }
        return returnList;
    }

    /**
     * 构建前端所需要下拉树结构
     * 
     * @param menus 菜单列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildMenuTreeSelect(List<SysMenu> menus)
    {
        List<SysMenu> menuTrees = buildMenuTree(menus);
        return menuTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 根据菜单ID查询信息
     * 
     * @param id 菜单ID
     * @return 菜单信息
     */
    @Override
    public SysMenu selectMenuById(Long id)
    {
        return menuMapper.selectMenuById(id);
    }

    /**
     * 是否存在菜单子节点
     * 
     * @param id 菜单ID
     * @return 结果
     */
    @Override
    public boolean hasChildById(Long id)
    {
        int result = menuMapper.hasChildById(id);
        return result > 0 ? true : false;
    }

    /**
     * 查询菜单使用数量
     * 
     * @param id 菜单ID
     * @return 结果
     */
    @Override
    public boolean checkMenuExistRole(Long id)
    {
        int result = roleMenuMapper.checkMenuExistRole(id);
        return result > 0 ? true : false;
    }

    /**
     * 新增保存菜单信息
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    @Override
    public int insertMenu(SysMenu menu)
    {
        return menuMapper.insertMenu(menu);
    }

    /**
     * 修改保存菜单信息
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    @Override
    public int updateMenu(SysMenu menu)
    {
        return menuMapper.updateMenu(menu);
    }

    /**
     * 删除菜单管理信息
     * 
     * @param id 菜单ID
     * @return 结果
     */
    @Override
    public int deleteMenuById(Long id)
    {
        return menuMapper.deleteMenuById(id);
    }

    /**
     * 校验菜单名称是否唯一
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    @Override
    public String checkMenuNameUnique(SysMenu menu)
    {
        Long id = StringUtils.isNull(menu.getId()) ? -1L : menu.getId();
        SysMenu info = menuMapper.checkMenuNameUnique(menu.getMenuName(), menu.getParentId());
        if (StringUtils.isNotNull(info) && info.getId().longValue() != id.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 获取路由名称
     * 
     * @param menu 菜单信息
     * @return 路由名称
     */
    public String getRouteName(SysMenu menu)
    {
        String routerName = StringUtils.capitalize(menu.getPath());
        // 非外链并且是一级目录（类型为目录）
        if (isMeunFrame(menu))
        {
            routerName = StringUtils.EMPTY;
        }
        return routerName;
    }

    /**
     * 获取路由地址
     * 
     * @param menu 菜单信息
     * @return 路由地址
     */
    public String getRouterPath(SysMenu menu)
    {
        String routerPath = menu.getPath();
        // 非外链并且是一级目录（类型为目录）
        if (0 == menu.getParentId().intValue() && UserConstants.TYPE_DIR.equals(menu.getMenuType())
                && UserConstants.NO_FRAME.equals(menu.getIsFrame()))
        {
            routerPath = "/" + menu.getPath();
        }
        // 非外链并且是一级目录（类型为菜单）
        else if (isMeunFrame(menu))
        {
            routerPath = "/";
        }
        return routerPath;
    }

    /**
     * 获取组件信息
     * 
     * @param menu 菜单信息
     * @return 组件信息
     */
    public String getComponent(SysMenu menu)
    {
        String component = UserConstants.LAYOUT;
        if (StringUtils.isNotEmpty(menu.getComponent()) && !isMeunFrame(menu))
        {
            component = menu.getComponent();
        }
        else if (StringUtils.isEmpty(menu.getComponent()) && isParentView(menu))
        {
            component = UserConstants.PARENT_VIEW;
        }
        return component;
    }

    /**
     * 是否为菜单内部跳转
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    public boolean isMeunFrame(SysMenu menu)
    {
        return menu.getParentId().intValue() == 0 && UserConstants.TYPE_MENU.equals(menu.getMenuType())
                && menu.getIsFrame().equals(UserConstants.NO_FRAME);
    }

    /**
     * 是否为parent_view组件
     * 
     * @param menu 菜单信息
     * @return 结果
     */
    public boolean isParentView(SysMenu menu)
    {
        return menu.getParentId().intValue() != 0 && UserConstants.TYPE_DIR.equals(menu.getMenuType());
    }

    /**
     * 根据父节点的ID获取所有子节点
     * 
     * @param list 分类表
     * @param parentId 传入的父节点ID
     * @return String
     */
    public List<SysMenu> getChildPerms(List<SysMenu> list, int parentId)
    {
        List<SysMenu> returnList = new ArrayList<SysMenu>();
        for (Iterator<SysMenu> iterator = list.iterator(); iterator.hasNext();)
        {
            SysMenu t = (SysMenu) iterator.next();
            // 一、根据传入的某个父节点ID,遍历该父节点的所有子节点
            if (t.getParentId() == parentId)
            {
                recursionFn(list, t);
                returnList.add(t);
            }
        }
        return returnList;
    }

    /**
     * 递归列表
     * 
     * @param list
     * @param t
     */
    private void recursionFn(List<SysMenu> list, SysMenu t)
    {
        // 得到子节点列表
        List<SysMenu> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SysMenu tChild : childList)
        {
            if (hasChild(list, tChild))
            {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysMenu> getChildList(List<SysMenu> list, SysMenu t)
    {
        List<SysMenu> tlist = new ArrayList<SysMenu>();
        Iterator<SysMenu> it = list.iterator();
        while (it.hasNext())
        {
            SysMenu n = (SysMenu) it.next();
            if (n.getParentId().longValue() == t.getId().longValue())
            {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<SysMenu> list, SysMenu t)
    {
        return getChildList(list, t).size() > 0 ? true : false;
    }


    /**
     * 代码生成业务表新增
     * 注意解决事务问题
     *
     * @param genTable
     * @return
     */
    @Override
    public AjaxResult insertGenTable(GenTable genTable) {
        //新增表成功的时候,新增字段公共字段
        int i = genTableMapper.insertGenTable(genTable);
        if (i > 0) {
            insertCommonColumn(genTable.getTableId());
            //判断是否是菜单
            isMenu(genTable);
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }

    /**
     * 新建表时新增对应公共字段
     *
     * @param tableId
     */
    public void insertCommonColumn(Long tableId) {

        String[] columnName = new String[]{"create_by", "create_time", "update_by", "update_time", "is_del"};
        String[] columnComment = new String[]{"创建者", "创建时间", "更新者", "更新时间", "是否删除"};
        String[] columnType = new String[]{"varchar(64)", "datetime", "varchar(64)", "datetime", "char(1)"};
        String[] htmlType = new String[]{"input", "datetime", "input", "datetime", "select"};
        for (int i = 0; i < columnName.length; i++) {
            GenTableColumn genTableColumn = new GenTableColumn();
            genTableColumn.setTableId(tableId);
            genTableColumn.setColumnName(columnName[i]);
            genTableColumn.setColumnComment(columnComment[i]);
            genTableColumn.setColumnType(columnType[i]);
            genTableColumn.setIsPk("0");
            genTableColumn.setIsIncrement("0");
            genTableColumn.setIsInsert("1");
            genTableColumn.setQueryType("EQ");
            genTableColumn.setHtmlType(htmlType[i]);
            genTableColumn.setSort(i + 1);
            genTableColumn.setCreateBy("admin");
            genTableColumn.setCreateTime(DateUtils.getNowDate());
            genTableColumnMapper.insertGenTableColumn(genTableColumn);
        }
    }

    //判断是否是菜单
    public void isMenu(GenTable genTable) {
        String isMenu = genTable.getIsMenu();
        if (StringUtils.isNotBlank(isMenu) && isMenu.equals(UserConstants.NOT_UNIQUE)) {
            //先生成主菜单
            SysMenu sysMenu = new SysMenu();
            //表描述当菜单名称
            sysMenu.setMenuName(genTable.getTableComment());
            sysMenu.setParentId(0L);
            sysMenu.setOrderNum("1");
            //路由地址
            sysMenu.setPath(genTable.getBusinessName());
            sysMenu.setComponent(genTable.getModuleName() + "/" + genTable.getTableComment() + "/index");
            sysMenu.setMenuType("C");
            sysMenu.setIcon("system");
            sysMenu.setCreateBy(SecurityUtils.getUsername());
            sysMenu.setTableName(genTable.getTableName());
            menuMapper.insertMenu(sysMenu);
            //是菜单,生成crud按钮
            for (int i = 0; i < Constants.MENU_NAME.length; i++) {
                String[] menuName = Constants.MENU_NAME;
                SysMenu menu = new SysMenu();
                menu.setParentId(sysMenu.getId());
                menu.setMenuName(menuName[i]);
                menu.setOrderNum(String.valueOf(i+1));
                menu.setMenuType("F");
                menu.setIcon("#");
                sysMenu.setCreateBy(SecurityUtils.getUsername());
                menuMapper.insertMenu(menu);
            }
        }
    }
}
